package tech.nevets.vcardgen;

import com.google.gson.Gson;
import com.google.gson.JsonObject;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;

import static spark.Spark.*;

public class Main {

    private static final Color WHITE = new Color(255, 255, 255);
    private static final Color GREEN = new Color(101, 142, 61);

    private static final Font ARIAL65 = new Font("Arial", Font.PLAIN, 65);
    private static final Font ARIAL55 = new Font("Arial", Font.PLAIN, 55);
    private static final Font ARIAL45 = new Font("Arial", Font.PLAIN, 45);
    private static final Font ARIAL45I = new Font("Arial", Font.ITALIC, 45);
    private static final Font ARIAL44I = new Font("Arial", Font.ITALIC, 44);
    private static final Font ARIAL40I = new Font("Arial", Font.ITALIC, 40);
    private static final Font ARIAL38I = new Font("Arial", Font.ITALIC, 38);

    public static void main(String[] args) {

        port(8080);
        get("/heartbeat", (req, res) -> {
            res.status(200);
            res.type("application/json");
            return "{ \"up\":true }";
        });
        post("/", (req, res) -> {
            res.type("image/png");
            JsonObject data = new Gson().fromJson(req.body(), JsonObject.class);
            return generate(
                    data.get("name").getAsString(),
                    data.get("title").getAsString(),
                    data.get("email").getAsString(),
                    data.get("location").getAsString(),
                    data.get("address").getAsString(),
                    data.get("schoolNumber").getAsString(),
                    data.get("extension").getAsString(),
                    data.get("hasExtension").getAsBoolean(),
                    data.get("cellNumber").getAsString(),
                    data.get("hasCell").getAsBoolean()
            );
        });

    }

    private static byte[] generate(String name, String title, String email, String location, String address, String schoolNumber, String extension, boolean hasExtension, String cellNumber, boolean hasCell) {
        BufferedImage rawImage = new BufferedImage(1080, 602, BufferedImage.TYPE_INT_ARGB);
        BufferedImage background;

        try {
            background = ImageIO.read(Main.class.getResource("/background.png"));
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }

        Graphics2D g = rawImage.createGraphics();
        g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

        g.drawImage(background, 0, 0, null);

        FontMetrics fm = g.getFontMetrics(ARIAL65);

        g.setColor(WHITE);
        if (fm.stringWidth(name) <= 969){
            g.setFont(ARIAL65);
        } else {
            g.setFont(ARIAL55);
        }
        g.drawString(name, 85, 112);

        fm = g.getFontMetrics(ARIAL45I);
        if (fm.stringWidth(title) <= 970) {
            g.setFont(ARIAL45I);
        } else {
            g.setFont(ARIAL40I);
        }
        g.drawString(title, 89, 176);

        boolean directLine = false;
        if (hasExtension && extension.length() == 0) {
            directLine = true;
        }

        int doubleNumOffset = 0;
        if (hasExtension && hasCell) {
            doubleNumOffset = 40;
        }

        int longAddrOffset = 0;
        fm = g.getFontMetrics(ARIAL38I);
        if (fm.stringWidth(address) > 694) {
            longAddrOffset = 40;
        }

        g.setFont(ARIAL45);
        g.drawString(email, 62, 380 - doubleNumOffset - longAddrOffset);

        g.setColor(GREEN);
        g.setFont(ARIAL44I);
        g.drawString(location, 59, 447 - doubleNumOffset - longAddrOffset);

        if (fm.stringWidth(address) >= 694) {

            StringBuilder addrLineOne = new StringBuilder();
            StringBuilder addrLineTwo = new StringBuilder();

            String[] splitAddr = address.split(",");
            int i = 0;
            for (int width = 0; width < 694; i++) {
                int splitSize = fm.stringWidth(splitAddr[i]);
                if ((width + splitSize) < 694) {
                    width += splitSize;
                    addrLineOne.append(splitAddr[i]);
                    addrLineOne.append(",");
                } else {
                    addrLineOne.deleteCharAt(addrLineOne.length() - 1);
                    break;
                }
            }

            for (; i < splitAddr.length; i++) {
                addrLineTwo.append(splitAddr[i]);
                addrLineTwo.append(",");
            }
            addrLineTwo.deleteCharAt(addrLineTwo.length() - 1);

            g.setFont(ARIAL38I);
            g.drawString(addrLineOne.toString().trim(), 59, 491 - doubleNumOffset - longAddrOffset);
            g.drawString(addrLineTwo.toString().trim(), 59, 491 - doubleNumOffset);
        } else {
            g.drawString(address, 59, 491 - doubleNumOffset);
        }

        String number = "";
        if (hasExtension && !hasCell) {
            if (directLine) {
                number = "W: " + schoolNumber;
            } else {
                number = "W: " + schoolNumber + " x" + extension;
            }
        } else if (!hasExtension && hasCell) {
            number = "C: " + cellNumber;
        }

        if (!(hasExtension && hasCell)) {
            g.drawString(number, 59, 536);
        } else {
            if (directLine) {
                g.drawString("W: " + schoolNumber, 59, 496);
            } else {
                g.drawString("W: " + schoolNumber + " x" + extension, 59, 496);
            }
            g.drawString("C: " + cellNumber, 59, 540);
        }

        g.dispose();

        BufferedImage finalImage = new BufferedImage(300, 167, BufferedImage.TYPE_INT_ARGB);


        g = finalImage.createGraphics();
        g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
        g.drawImage(rawImage, 0, 0, 300, 167, null);
        g.dispose();

        ByteArrayOutputStream baos;
        try {
            baos = new ByteArrayOutputStream();
            ImageIO.write(finalImage, "png", baos);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }

        return baos.toByteArray();
    }
}
